# %% 
import numpy as np
import pandas
import matplotlib.pyplot as plt
from Schroen2017hess import Wr, D86, Weight_d, get_footprint

# %%
# input values

class Profile:
    __slots__ = [
        # Input
        "sm",   # soil moisture values
        "d",    # depth values in cm
        "bd",   # bulk density 
        "r",    # distance from the CRNS in m
        # Calculated
        "D86",  # penetration depth
        "w_r",  # radial weight of this profile
        "sm_wavg", # vertically weighted average sm
        "data"  # DataFrame
        ]
    
    def __init__(self, sm, d, bd, r=1):
        
        # Vector data
        self.sm = np.array(sm)
        self.d  = np.array(d)
        self.bd = np.array(bd)
        self.data = None
        self.update_data()
        
        # Scalar values
        self.r  = r
        self.D86 = np.nan
        self.sm_wavg = np.nan
        self.w_r     = np.nan
        
    
    def update_data(self):
        if not self.data:
            self.data = pandas.DataFrame()
        self.data["sm"] = self.sm
        self.data["d"]  = self.d
        self.data["bd"] = self.bd
        if not "weight" in self.data.columns:
            self.data["weight"] = np.nan

# %%
# Site
air_humidity = 5
air_pressure = 1013

profiles = [
    Profile(
        sm = [0.21, 0.42, 0.23, 0.19],
        d  = [   5,   10,   20,   40],
        bd = [ 1.5,  1.5,  1.5,  1.5],
        r  =  5
    ),
    Profile(
        sm = [0.11, 0.12, 0.13, 0.09],
        d  = [   5,   10,   20,   40],
        bd = [ 1.5,  1.5,  1.5,  1.5],
        r  = 55
    ),
    Profile(
        sm = [0.31, 0.33, 0.33, 0.49],
        d  = [   5,   10,   20,   40],
        bd = [ 1.5,  1.5,  1.5,  1.5],
        r  = 95
    )
]

print("Set up %.0f profiles." % len(profiles))

# %%
i = 0
for P in profiles:
    
    i += 1
    # First order estimate of the average soil moisture
    sm_estimate  = P.sm.mean()

    # Neutron penetration depth at this location
    P.D86 = D86(
        sm = P.sm.mean(),
        bd = P.bd.mean(),
        r  = P.r
    )

    # Weights for this profile
    # data = P.data.copy()
    P.data["weight"] = Weight_d(
        P.data["d"],
        P.D86
    )
    # Calculate weighted sm average
    P.sm_wavg = np.average(
        P.data["sm"],
        weights = P.data["weight"]
    )

    # Calculate horizontal weight for this profile
    P.w_r = Wr(
        r  = P.r,
        sm = P.sm_wavg,
        hum= air_humidity
    )

    print("Profile %.0f: vertical wt. avg. SM: %.3f, CRNS depth: %3.0f cm, distance: %2.0f m, horizontal weight: %7.0f" \
          % (i, P.sm_wavg, P.D86, P.r, P.w_r))

# %%
# Horizontal average
profiles_sm_wavg = [P.sm_wavg for P in profiles]
profiles_weights = [P.w_r     for P in profiles]
sm_wavg = np.average(
    profiles_sm_wavg,
    weights = profiles_weights
)
print("Horizontal wt. avg. SM: %.3f" % sm_wavg)

# %%
# Footprint
footprint_m = get_footprint(sm_wavg, air_humidity, air_pressure)
print("Footprint radius: %.0f m" % footprint_m)

# %%
# Plot vertical profiles

plt.title("SM profiles, vertical wt. avg. SM, and CRNS depth")
for P in profiles:
    plt.plot(P.sm, P.d, "o-")
    plt.plot(
        [P.sm_wavg, P.sm_wavg],
        [0, P.D86],
        "v--", color="black", ms=10
    )
plt.xlim(0,1)
plt.ylim(60,0)
plt.xlabel("SM")
plt.ylabel("Depth (cm)")
plt.show()

# %%
# Plot horizontal profiles

plt.title("SM profile locations and vertical wt. avg. SM,\nhorizontal wt. avg. SM, CRNS footprint")
for P in profiles:
    plt.plot(P.r, P.sm_wavg, "o-")
    plt.plot(
        [0, footprint_m],
        [sm_wavg, sm_wavg],
        ">--", color="black", ms=10
    )
plt.xlim(0,250)
plt.ylim(0.0,1.0)
plt.xlabel("Distance (m)")
plt.ylabel("SM")
plt.show()



# %%
